/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/*
 * anti.c
 * This program draws two intersecting cubes and uses the 
 * accumulation buffer to do scene anti-aliasing.
 *
 *		SPACE key		- toggle antialiasing
 *		Escape key		- exit the program
 *
 * Note: this program must be compiled with accpersp.c and jitter.h
 */

#include <GL/gl.h>
#include <GL/glu.h>
#include <stdio.h>
#include <math.h>
#include "aux.h"
#include "jitter.h"

GLvoid initLighting(GLvoid);
GLvoid initialize(GLvoid);
GLvoid resize( GLsizei width, GLsizei height );
GLvoid drawScene(GLvoid);
GLvoid toggleJitter(GLvoid); 
GLvoid printHelp( char * );

GLvoid accFrustum(GLdouble, GLdouble, GLdouble, GLdouble,
	GLdouble, GLdouble, GLdouble, GLdouble, 
	GLdouble, GLdouble, GLdouble);
GLvoid accPerspective(GLdouble, GLdouble, GLdouble, GLdouble,
	GLdouble, GLdouble, GLdouble, GLdouble, GLdouble);

GLint		maxJitters = 1;
GLfloat		aspect = 1.0;

GLvoid
main(int argc, char *argv[] )
{
	initialize();

	printHelp( argv[0] );

	auxMainLoop( drawScene );
} 

GLvoid
printHelp( char *progname )
{
	fprintf(stdout, "\n%s - demonstrates using the \n"\
		"accumulation buffer to antialias an entire scene.\n"\
		"Expect it to run very slowly on systems without \n"\
		"a hardware accumulation buffer.\n\n"\
		"Escape key		- exit the program\n"\
		"SPACE key		- toggle scene antialiasing\n\n", 
		progname );

	printf("scene anti-aliasing is OFF\n");
}

GLvoid  
toggleJitter(GLvoid)
{

	static GLboolean jitterFlag = GL_FALSE;

	jitterFlag = !jitterFlag;

	if ( jitterFlag )
		maxJitters = 8;
	else
		maxJitters = 1;

	printf("scene anti-aliasing is %s\n", 
		(jitterFlag?"ON":"OFF"));
}


GLvoid
resize( GLsizei width, GLsizei height )
{
	glViewport( 0, 0, width, height );
	aspect = (GLfloat) width/(GLfloat) height ;

        glMatrixMode( GL_PROJECTION );
        glLoadIdentity();
        glMatrixMode( GL_MODELVIEW );
}

GLvoid
initLighting(GLvoid)
{
	GLfloat shiny_mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
	GLfloat shiny_mat_shininess[] = { 128.0 };
    
	glMaterialfv(GL_FRONT, GL_SPECULAR, shiny_mat_specular); 
	glMaterialfv(GL_FRONT, GL_SHININESS, shiny_mat_shininess); 
	
	glEnable(GL_DEPTH_TEST);

	glShadeModel (GL_FLAT);

	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHTING);

	glColorMaterial(GL_FRONT, GL_DIFFUSE);
}

GLvoid
initialize(GLvoid)
{

	GLsizei  width, height, winSize;

	auxGetScreenSize( &width, &height );
	winSize = ( width < height ) ? width : height;
	auxInitPosition(winSize/4, winSize/4, winSize/4, winSize/4);
	auxInitDisplayMode(AUX_RGBA|AUX_DEPTH|AUX_DOUBLE|AUX_ACCUM);
	auxInitWindow( "Anti Aliasing" );
	
	auxAnimation( AUX_ON );

	auxReshapeFunc( resize );

	auxKeyFunc(AUX_ESCAPE, auxQuit);
	auxKeyFunc(AUX_SPACE, toggleJitter);

	glClearColor( 0.0, 0.0, 0.0, 1.0 );
	glClearAccum( 0.0, 0.0, 0.0, 0.0 );

	glEnable(GL_DEPTH_TEST);

	initLighting();
}

/* Draws two cubes with scene anti-aliasing */
GLvoid
drawScene(GLvoid)
{
	int jitter;
	static GLfloat angle = 0.0;
	
	if ( maxJitters > 1 )
		glClear(GL_ACCUM_BUFFER_BIT);

	for(jitter = 0; jitter < maxJitters; jitter++)
	{
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		glPushMatrix();
		        accPerspective(45.0, aspect,
				1.0, 15.0, j8[jitter].x, j8[jitter].y, 
				0.0, 0.0, 1.0);
			glTranslatef (0.0, 0.0, -8.0); 

			glRotatef(angle, 1.0, 0.0, 0.0);
	 
			glEnable(GL_COLOR_MATERIAL);
			glColor3f(0.9, 0.9, 0.9);
			auxSolidCube( 1.0 );

			glRotatef(45.0, 0.0, 1.0, 0.0); 
			glRotatef(45.0, 0.0, 0.0, 1.0);
	 
			glColor3f(0.9, 0.0, 0.9);
			auxSolidCube( 1.0 );
			glDisable(GL_COLOR_MATERIAL);
		glPopMatrix();

		if ( maxJitters > 1 )
			glAccum(GL_ACCUM,  1.0/maxJitters);
	}

	if ( maxJitters > 1 )
		glAccum(GL_RETURN,  1.0);

	angle += 5.0;
	auxSwapBuffers();
}
