/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/*
 *  blur.c demonstrates using the accumulation buffer for
 *  effects like motion blur.
 * 
 *            Escape key               - exit the program
 */

#include <aux.h>
#include <math.h>
#include <stdio.h>

/* Function Prototypes */

GLvoid  initialize( GLvoid );
GLvoid  drawScene( GLvoid );
GLvoid  resize( GLsizei, GLsizei );
GLvoid  startMove( AUX_EVENTREC * );
GLvoid  stopMove( AUX_EVENTREC * );
GLvoid  mouseMove( AUX_EVENTREC * );
GLvoid  printHelp( GLvoid );
GLvoid  initLighting( GLvoid );

/* Global Variables */
GLfloat     latitude, longitude, radius;
GLint       xStart, yStart;
GLboolean   moving = GL_FALSE;

GLvoid
main()
{
	initialize();
	auxMainLoop( drawScene );
}

GLvoid
initialize( GLvoid )
{
	GLsizei    width, height, winSize;

	auxGetScreenSize( &width, &height );
	winSize = ( width < height ) ? width : height;
	auxInitPosition( winSize / 4, winSize / 4, winSize / 8,
			winSize / 8 );
	auxInitDisplayMode(AUX_RGBA|AUX_DEPTH|AUX_DOUBLE|AUX_ACCUM);
	auxInitWindow( "blur" );

	auxReshapeFunc( resize );

	auxKeyFunc( AUX_ESCAPE, auxQuit );

	/* Set up functions using the mouse */
	auxMouseFunc( AUX_LEFTBUTTON, AUX_MOUSEDOWN, startMove );
	auxMouseFunc( AUX_LEFTBUTTON, AUX_MOUSEUP, stopMove );
	auxMouseFunc( AUX_MOUSELOC, 0, mouseMove );

	glClearColor( 0.0, 0.0, 0.0, 1.0 );
	glClearDepth( 1.0 );
	glEnable( GL_DEPTH_TEST );
	glEnable( GL_CULL_FACE );

	glMatrixMode( GL_PROJECTION );
	gluPerspective( 45.0, 1.0, 3.0, 7.0 );
	glMatrixMode( GL_MODELVIEW );

	radius = 4.5;
	latitude = 25.0;
	longitude = 0.0;
	auxAnimation( AUX_ON );
		
	initLighting();
	printHelp();

	glClearAccum(0.0, 0.0, 0.0, 0.0);
	glClear( GL_ACCUM_BUFFER_BIT );

}

/*  Initialize material property, light source, lighting model, 
 *  and depth buffer.
 */
GLvoid
initLighting( GLvoid )
{
	static GLfloat mat_color[] = { 0.3, 0.3, 0.3, 1.0 };

	glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_color );
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
}

GLvoid
printHelp( GLvoid )
{
	fprintf(stdout, 
"\nblur\n\n\\
Escape key               - exit the program\n\
Left Mousebutton, down   - start moving the eye\n\
Left Mousebutton, up     - stop moving the eye\n\
Mouse pointer location   - position eye\n\n");
}

/* Define user input functions */
GLvoid
startMove( AUX_EVENTREC *event )
{
	moving = GL_TRUE;

	/* Record where the mouse pointer was when the user pressed 
	 * the left mouse buton 
	 */
	xStart = event->data[AUX_MOUSEX];
	yStart = event->data[AUX_MOUSEY];
}

GLvoid
stopMove( AUX_EVENTREC *event )
{
	moving = GL_FALSE;
}

GLvoid
mouseMove( AUX_EVENTREC *event )
{
	GLint     xPos, yPos;
	GLint     xOffset, yOffset;

	if ( moving ) {
		xPos = event->data[AUX_MOUSEX];
		yPos = event->data[AUX_MOUSEY];

		/* Using the stored and current mouse positions, 
		 * compute how far the mouse moved 
		 */
		xOffset = xPos - xStart;
		yOffset = yPos - yStart;

		/* Adjust the eye position based on the mouse position */
		longitude -= xOffset;
		latitude += yOffset;

		/* Update the stored mouse position for later use */
		xStart = xPos;
		yStart = yPos;
	}
}

GLvoid
resize( GLsizei width, GLsizei height )
{
	GLfloat     aspect;

	glViewport( 0, 0, width, height );

	aspect = (GLfloat) width / height;

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0, aspect, 3.0, 7.0 );
	glMatrixMode( GL_MODELVIEW );

	glClear( GL_ACCUM_BUFFER_BIT );
}

GLvoid
drawScene( GLvoid )
{
	static float spin = 0.0;

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	glPushMatrix();

		glTranslatef( 0.0, 0.0, -radius);
		glRotatef( latitude, 1.0, 0.0, 0.0 );
		glRotatef( longitude, 0.0, 0.0, 1.0 );

		glPushMatrix();
			glAccum(GL_MULT, 0.5);
			glRotatef( spin, 0.0, 1.0, 0.0 );
			glTranslatef( 1.0, 0.0, 0.0 );
			auxSolidSphere( 0.5 );
			glAccum(GL_ACCUM, 1.0);
			glAccum(GL_RETURN, 1.0);
		glPopMatrix();

	glPopMatrix();

	auxSwapBuffers();

	spin = fmodf( spin + 10.0, 360.0 );
}
