/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/* dof.c
 *	This program demonstrates using the accumlation buffer to 
 *	simulate depth-of-field.  Objects at the focal point are
 *	crisp, the rest are somewhat fuzzy.
 *
 *		Escape key		- exit the program
 *		SPACE key		- toggle depth-of-field
 */

#include <aux.h>
#include <math.h>
#include <stdio.h>
#include "jitter.h"

/*  Function Prototypes  */

GLvoid		initialize();
GLvoid		drawScene( GLvoid );
GLvoid		resize( GLsizei width, GLsizei height );
GLvoid		toggleJitter(GLvoid); 
GLvoid		initLighting( GLvoid );

GLvoid accFrustum(GLdouble, GLdouble, GLdouble, GLdouble,
	GLdouble, GLdouble, GLdouble, GLdouble, 
	GLdouble, GLdouble, GLdouble);
GLvoid accPerspective(GLdouble, GLdouble, GLdouble, GLdouble,
	GLdouble, GLdouble, GLdouble, GLdouble, GLdouble);

void printHelp( char * );

GLint		maxJitters = 1;
GLfloat		aspect = 1.0;

void
main(int argc, char *argv[])
{
	initialize();
	printHelp(argv[0]);
	auxMainLoop( drawScene );
}

void
printHelp( char *progname )
{
	fprintf(stdout, "\n%s - demonstrates using the \n"\
		"accumulation buffer to simulate depth of field.\n"\
		"Expect it to run very slowly on systems without \n"\
		"a hardware accumulation buffer.\n\n"\
		"Escape key		- exit the program \n"\
		"SPACE key		- toggle depth-of-field\n\n",
		progname );

	printf("depth-of-field is OFF\n");
}

GLvoid  
toggleJitter(GLvoid)
{

	static GLboolean jitterFlag = 0;

	jitterFlag = !jitterFlag;

	if ( jitterFlag )
		maxJitters = 8;
	else
		maxJitters = 1;

	printf("depth-of-field is %s\n", (jitterFlag?"ON":"OFF"));
}

GLvoid
resize( GLsizei width, GLsizei height )
{

	glViewport( 0, 0, width, height );

	aspect = (GLfloat) width / height;
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	glMatrixMode( GL_MODELVIEW );
}

GLvoid
initialize( )
{
	GLsizei    width, height;

	auxGetScreenSize( &width, &height );
	auxInitPosition( width / 4, height / 4, width / 4,
		height / 4 );
	auxInitDisplayMode( AUX_RGBA | AUX_DEPTH | AUX_DOUBLE | AUX_ACCUM );
	auxInitWindow( "dof" );
	auxAnimation( AUX_ON );
	
	auxReshapeFunc( resize );
	auxKeyFunc( AUX_ESCAPE, auxQuit );
	auxKeyFunc( AUX_SPACE, toggleJitter );
	
	glClearColor( 0, 0, 0, 1 );
	glClearAccum( 0.0, 0.0, 0.0, 0.0 );

	glEnable( GL_DEPTH_TEST );

	aspect = (GLfloat) width / height;

	initLighting(); 
}

/*  Initialize material property, light source, lighting model, 
 *  and depth buffer.
 */
GLvoid
initLighting( GLvoid )
{
	GLfloat mat_specular[] = { 0.8, 0.8, 0.8, 1.0 };
	GLfloat mat_shininess[] = { 128.0 };
	GLfloat light_position[] = { 1.0, 1.0, 1.0, 0.0 };
	GLfloat lmodel_ambient[] = { 0.2, 0.2, 0.2, 1.0 };

	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
	glShadeModel( GL_SMOOTH );
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glColorMaterial(GL_FRONT, GL_DIFFUSE);
}

/* When maxJitters is greater than 1, drawScene() draws 5 torii
 * into the accumulation buffer 8 times; each time with a 
 * jittered perspective.
 *
 * The torii are drawn at +4,+2,0,2,-4 in z from the focal plane, 
 * so the third torii will stay in focus.  
 * The amount of jitter is adjusted by aperture times the 
 * magnitude of the accPerspective() jitter (see jitter.h)
 */
GLvoid
drawScene( GLvoid )
{
        int i, jitter, torii = 5;
        static GLfloat focalPlane = 10.0, aperture = 0.5;

        if (maxJitters > 1)
                glClear(GL_ACCUM_BUFFER_BIT);
        for (jitter = 0; jitter < maxJitters; jitter++) {
                glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
                glPushMatrix();
                        accPerspective (45.0, aspect, 1.0, 15.0, 0.0, 0.0,
                                aperture*j8[jitter].x, aperture*j8[jitter].y, 
                                focalPlane);
                        glEnable(GL_COLOR_MATERIAL);
                        for ( i = 0; i < torii; i++ )
                        {
                                glColor3f(i/5.0, i/5.0, 1.0 - i/5.0); 
                                glPushMatrix(); 
                                        /* draw 5 torii at +4,+2,0,-2,-4
                                         * from focalPlane in z.
                                         */
                                        glTranslatef( -1.0 + i, 0.0,
                                                -focalPlane + (4.0 - i*2.0) );
                                        auxSolidTorus( 0.25, 0.75 );
                                glPopMatrix();
                        }
                        glDisable(GL_COLOR_MATERIAL);
                glPopMatrix();
                if (maxJitters > 1)
                        glAccum (GL_ACCUM, 1/(GLfloat) maxJitters);
        }
        if (maxJitters > 1)
                glAccum (GL_RETURN, 1.0);
        auxSwapBuffers();
}
