/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/*
 *  spotlightAlpha.c - spotlight shining on a stack of flat alpha 
 *	blended grids
 *
 *	Escape key		- exit the program
 *	<z> key			- turn depth buffering on/off
 *	Left Mousebutton, down	- start moving the eye
 *	Left Mousebutton, up	- stop moving the eye
 *	Mouse pointer location	- position eye
 *	SPACEBAR key		- wireframe/lit/smooth grid
 */

#include <aux.h>
#include <math.h>
#include <stdio.h>

#define GRIDPOINTS 41
#define WIRE 0
#define LIT_FLAT 1
#define LIT_SMOOTH 2
#define POINTS 3

/*  Function Prototypes  */

GLvoid  initialize( GLvoid );
GLvoid  drawScene( GLvoid );
GLvoid  resize( GLsizei, GLsizei );
GLvoid  initGrid( GLvoid );
GLvoid  drawGridWire( GLvoid );
GLvoid  drawGridFilled( GLvoid );
GLvoid  setDrawStyle( GLvoid );
GLvoid  startMove( AUX_EVENTREC * );
GLvoid  stopMove( AUX_EVENTREC * );
GLvoid  mouseMove( AUX_EVENTREC * );
GLvoid  printHelp( GLvoid );
GLvoid  initLighting( GLvoid );
GLvoid  toggleDepthBuffer( GLvoid );

/* Global Variables */
GLfloat     latitude, longitude, radius;
GLint       xStart, yStart;
GLint       drawStyle = LIT_SMOOTH;
GLboolean   moving = GL_FALSE;
GLboolean   depthFlag = GL_FALSE;
GLfloat     grid[GRIDPOINTS][GRIDPOINTS][3] = { 0 };


GLvoid
main()
{
	initialize();

	auxMainLoop( drawScene );
}

/*  Initialize material property, light source, lighting model, 
 *  and depth buffer.
 */
GLvoid
initLighting( GLvoid )
{
	GLfloat mat_ambient[] = { 0.0, 0.0, 0.0, 0.0 };
	GLfloat mat_diffuse[] = { 1.0, 1.0, 1.0, 0.5 };

	glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
	glShadeModel( GL_SMOOTH );
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
}


GLvoid
initialize( GLvoid )
{
	GLsizei    width, height, winSize;

	auxGetScreenSize( &width, &height );
	winSize = ( width < height ) ? width : height;
	auxInitPosition( winSize / 4, winSize / 4, winSize / 2,
			winSize / 2 );

	auxInitDisplayMode( AUX_RGBA | AUX_DEPTH | AUX_DOUBLE );

	auxInitWindow( "spotlightAlpha  'z' key//SPACEBAR - toggle depth bufring//filled/smooth/wireframe grid" );

	auxReshapeFunc( resize );

	auxKeyFunc( AUX_ESCAPE, auxQuit );
	auxKeyFunc( AUX_SPACE, setDrawStyle );
	auxKeyFunc( AUX_z, toggleDepthBuffer );

	/* Set up functions using the mouse */
	auxMouseFunc( AUX_LEFTBUTTON, AUX_MOUSEDOWN, startMove );
	auxMouseFunc( AUX_LEFTBUTTON, AUX_MOUSEUP, stopMove );
	auxMouseFunc( AUX_MOUSELOC, 0, mouseMove );

	glClearColor( 0.0, 0.0, 0.0, 1.0 );
	glClearDepth( 1.0 );

	glMatrixMode( GL_PROJECTION );
	gluPerspective( 45.0, 1.0, 3.0, 7.0 );
	glMatrixMode( GL_MODELVIEW );

	glShadeModel( GL_FLAT );

	radius = 4.5;
	latitude = 25.0;
	longitude = 0.0;
	auxAnimation( AUX_ON );
		
	initLighting();
	initGrid();

	printHelp();
}

GLvoid
printHelp( GLvoid )
{
	fprintf(stdout, 
"\nspotlightAlpha\n\n\\
Escape key		- exit the program\n\
<z> Key			- toggle depth (z) buffering\n\
Left Mousebutton, down	- start moving the eye\n\
Left Mousebutton, up	- stop moving the eye\n\
Mouse pointer location	- position eye\n\
SPACEBAR		- filled/smooth/wireframe grid\n\n");
}

/* Define user input functions */
GLvoid
setDrawStyle( GLvoid )
{
	if (++drawStyle > POINTS)
		drawStyle = WIRE;

	switch (drawStyle)
	{
		case WIRE:
		case POINTS:
			glEnable(GL_LIGHTING);
			glEnable(GL_LIGHT0);
			break;
		case LIT_FLAT:
			glShadeModel( GL_FLAT );
			glEnable(GL_LIGHTING);
			glEnable(GL_LIGHT0);
			break;
		case LIT_SMOOTH:
			glShadeModel( GL_SMOOTH );
			glEnable(GL_LIGHTING);
			glEnable(GL_LIGHT0);
			break;
		default:
			break;
	}
}

GLvoid
startMove( AUX_EVENTREC *event )
{
	moving = GL_TRUE;

	/* Record where the mouse pointer was when the user pressed the
	 *    left mouse buton */
	xStart = event->data[AUX_MOUSEX];
	yStart = event->data[AUX_MOUSEY];
}

GLvoid
stopMove( AUX_EVENTREC *event )
{
	moving = GL_FALSE;
}

GLvoid
mouseMove( AUX_EVENTREC *event )
{
	GLint     xPos, yPos;
	GLint     xOffset, yOffset;

	if ( moving ) {
		xPos = event->data[AUX_MOUSEX];
		yPos = event->data[AUX_MOUSEY];

		/* Using the stored and current mouse positions, compute how far
		 *    the mouse moved */
		xOffset = xPos - xStart;
		yOffset = yPos - yStart;

		/* Adjust the eye position based on the mouse position */
		longitude -= xOffset;
		latitude += yOffset;

		/* Update the stored mouse position for later use */
		xStart = xPos;
		yStart = yPos;
	}
}

GLvoid  
toggleDepthBuffer( GLvoid )
{
	depthFlag = !depthFlag;
	if (depthFlag) {
		glEnable( GL_DEPTH_TEST );
		printf("Depth test ON\n");
	} else {
		glDisable( GL_DEPTH_TEST );
		printf("Depth test OFF\n");
	}
}


GLvoid
resize( GLsizei width, GLsizei height )
{
	GLfloat     aspect;

	glViewport( 0, 0, width, height );

	aspect = (GLfloat) width / height;

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0, aspect, 3.0, 7.0 );
	glMatrixMode( GL_MODELVIEW );
}


GLvoid
drawScene( GLvoid )
{
	int i;
	static float spin = 0.0;
		
	GLfloat light_position[] = { 0.0, 2.5, 0.0, 1.0 };
	GLfloat spot_direction[] = { 0.1, -1.0, 0.0 };
	GLfloat spot_cutoff[] = { 10.0 };
	GLfloat spot_exponent[] = { 128.0 };

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	glPushMatrix();

	glTranslatef( 0.0, 0.0, -radius);
	glRotatef( latitude, 1.0, 0.0, 0.0 );
	glRotatef( longitude, 0.0, 0.0, 1.0 );

	glPushMatrix();
		glRotatef( spin, 0.0, 1.0, 0.0 );
		glLightfv(GL_LIGHT0, GL_SPOT_DIRECTION, spot_direction);
		glLightfv(GL_LIGHT0, GL_SPOT_CUTOFF, spot_cutoff);
		glLightfv(GL_LIGHT0, GL_SPOT_EXPONENT, spot_exponent);
		glLightfv(GL_LIGHT0, GL_POSITION, light_position);
	glPopMatrix();

	for( i = -2; i < 3; i++)
	{
		glPushMatrix();
		glTranslatef( 0.0, i * 0.5, 0.0);
		if (drawStyle == WIRE)
			drawGridWire();
		else
			drawGridFilled();
		glPopMatrix();
	}
	glPopMatrix();

	auxSwapBuffers();

	glFlush();
	
	spin = fmodf( spin + 3.0, 360.0 );
}

GLvoid
initGrid( GLvoid )
{
	int i, j;
	GLfloat     x, z, spacing;

	/* Create a flat grid composed of points
	 * with the y value = 0
	 */
	
	/* spacing = distance between adjacent grid points in x and z */
	spacing = 2.0 / (float)(GRIDPOINTS - 1);

	/* x and z range from -1.0 to 1.0 by spacing interval */
	for ( x = -1.0, i = 0; i < GRIDPOINTS; x += spacing, i++ ) {
		for ( z = -1.0, j = 0; j < GRIDPOINTS; z += spacing, j++ ) {
			grid[i][j][0] = x;
			grid[i][j][1] = 0.0;
			grid[i][j][2] = z;
		}
	}
}

GLvoid
drawGridWire( GLvoid )
{
	register int i, j;

	/* draw vertical lines connecting grid points */
	for ( i = 0; i < (GRIDPOINTS); i++ ) {
		glBegin( GL_LINE_STRIP );
		for ( j = 0; j < (GRIDPOINTS); j++ ) {
			glVertex3fv( &grid[i][j][0] );
		}
		glEnd();
	}
	/* draw horizontal lines connecting grid points */
	for ( j = 0; j < (GRIDPOINTS); j++ ) {
		glBegin( GL_LINE_STRIP );
		for ( i = 0; i < (GRIDPOINTS); i++ ) {
			glVertex3fv( &grid[i][j][0] );
		}
		glEnd();
	}
}


GLvoid
drawGridFilled( GLvoid )
{
	register int i, j;

	glNormal3f( 0.0, 1.0, 0.0 );
	
	/* Draw quad strips connecting grid points. 
	 * Grid is drawn column by column, 
	 * where each column is one quad strip.
	 * Every pair of vertices after first two vertices
	 * adds a new quad to the strip.
	 */
	for ( i = 0; i < (GRIDPOINTS - 1); i++ ) {
		if (drawStyle == POINTS)
			glBegin( GL_POINTS );
		else
			glBegin( GL_QUAD_STRIP );
		for ( j = 0; j < (GRIDPOINTS); j++ ) {
			glVertex3fv( &grid[i][j][0] );
			glVertex3fv( &grid[i + 1][j][0] );				
		}
		glEnd();
	}
}

