/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/*
 * dlist.c uses the code in mountain.c to draw fractal-based 
 * mountains.
 *
 * A simple benchmarking loop runs for MAX_FRAMES then prints 
 * the average frames/second.  
 * 
 * If no argument is given, the loop is run in immediate mode.
 * If any argument is given, the loop is run in display list mode.  
 *
 * If BENCHMARK_SINGLEBUFFER is defined, benchmarking will be 
 * performed in single buffer mode.  
 */
#include <aux.h>
#include <math.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <stdlib.h>

/*  Function Prototypes  */

GLvoid initialize( int, char *[] );
GLvoid drawScene( GLvoid );
GLvoid perspectiveViewing( GLfloat );
GLvoid resize( GLsizei, GLsizei );
GLvoid initLighting( GLvoid );
GLvoid makeMountainLists( GLvoid );

GLvoid resetView( GLvoid );
GLvoid computeMountain( GLvoid );
GLvoid drawMountain( GLvoid );

void   printHelp( char *progname );
double getTime( void );

GLint mountainBase;

GLboolean displayListMode = GL_FALSE;

#define MAX_LISTS 10
#define MAX_FRAMES 120

void
main( int argc, char *argv[] )
{
	initialize( argc, argv );
	auxMainLoop( drawScene );
}

void
printHelp( char *progname )
{
	fprintf( stdout, 
		"\n%s contains a simple benchmarking loop that \n"\
		"generates fractal-based mountains.\n"\
		"It creates %d frames, then prints "\
		"the average frames/second.\n"\
		"If any argument is specified, it runs in display "\
		"list mode, \notherwise it runs in immediate mode\n\n",
		progname, MAX_FRAMES );
}

GLvoid
resize( GLsizei width, GLsizei height )
{
	GLfloat     aspect;
	glViewport( 0, 0, width, height );

	aspect = (GLfloat) (width) / height;
	perspectiveViewing( aspect );
}


GLvoid
perspectiveViewing( GLfloat aspect )
{
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0, aspect, 1.0, 350.0 );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
}

GLvoid
initialize( int argc, char *argv[] )
{
	GLint winSize, width, height;

	auxGetScreenSize( &width, &height );
	winSize = ( width < height ) ? width : height;
	auxInitPosition( winSize / 4, winSize / 4, winSize/2, 
		winSize/2 );
#ifdef BENCHMARK_SINGLEBUFFER           
	auxInitDisplayMode( AUX_RGBA | AUX_DEPTH );
#else
	auxInitDisplayMode( AUX_RGBA | AUX_DOUBLE | AUX_DEPTH );
#endif
	auxInitWindow( "dlist      Benchmark" );

	auxAnimation( AUX_ON );

	auxReshapeFunc( resize );

	auxKeyFunc( AUX_ESCAPE, auxQuit );

	glClearColor( 0, 0, 0, 1 );
	glClear ( GL_COLOR_BUFFER_BIT );
	perspectiveViewing( 1.0 );

	resetView();

	glEnable(GL_DEPTH_TEST);

	initLighting();

	printHelp( argv[0] );

	if ( argc > 1 ) {
		displayListMode = GL_TRUE;
		auxSwapBuffers();

		makeMountainLists();

		fprintf(stdout, 
			"Starting Display List Mode Benchmark\n");
		fprintf(stdout, "Display List Mode results:\t");
	} else {
		fprintf(stdout, "Starting Immediate Mode Benchmark\n");
		fprintf(stdout, "Immediate Mode results:\t");
	}
}


/*
 * makeMountainLists() creates MAX_LISTS number of mountains and stores
 * each in a display list.
 */
GLvoid
makeMountainLists(GLvoid)
{

	int mountain;

	/* reset the seed so that the sequence of mountains generated
	 * will be repeatable.
	 */
	srand(1);

	fprintf( stdout, "Initializing %d Display Lists: ", 
		MAX_LISTS );

	mountainBase = glGenLists( MAX_LISTS );
	for ( mountain = 0; mountain < MAX_LISTS; mountain++ )
	{
		fprintf( stdout, "%d ", mountain );
		fflush( stdout );

		glNewList( mountainBase + mountain, GL_COMPILE );
			computeMountain();
			drawMountain();
		glEndList();
	}

	fprintf( stdout, "Done.\n\n" );
}

GLvoid 
initLighting( GLvoid )
{
	static GLfloat darkGreen[] = { 0.0, 0.5, 0.0, 1.0 };

	glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, darkGreen );

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
}

GLvoid
drawScene( GLvoid )
{
	static int mountain = 0;
	static int frameCount = 0;

	static double t0;
	double elapsed;

	if (!frameCount)
	{
		t0 = getTime();
	}

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	if ( displayListMode )
		glCallList( mountainBase + mountain);
	else
	{
		computeMountain();
		drawMountain();
	}

#ifndef BENCHMARK_SINGLEBUFFER          
	auxSwapBuffers();
#else
	glFlush();
#endif
	/* cycle through MAX_LISTS different mountains */
	mountain = ++frameCount % MAX_LISTS;
	if (mountain == 0)
	{
		/* reset seed to initial value so that we display
		 * the same set of mountains in immediate mode
		 * as in display list mode
		 */
		srand(1);
	}

	if (frameCount == (MAX_FRAMES) )
	{
		glFinish();
		elapsed = getTime() - t0;
		fprintf ( stdout, "Average %3.1lf frames/second\n", 
		    MAX_FRAMES/elapsed );
		auxQuit();
	}
}
double
getTime( void )
{
	struct timeval tp;
	struct timezone tzp;
	long sec,usec;

	gettimeofday(&tp,&tzp);
	sec = tp.tv_sec;
	usec = tp.tv_usec;
	return ((double)sec + (double)usec/1000000);
}
