/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/* koosh.c 
 * This program draws a koosh ball.  When various mouse buttons are 
 * pressed, the properties of the koosh ball and lighting are 
 * modified.  The properties of the koosh ball are kept in display
 * lists which are in turn called by the kooshball displaylist.
 *
 *	Escape key			- exit the program
 *	Left Mousebutton, down		- change diffuse material color
 *	Middle Mousebutton, down 	- toggle infinite/local light model
 *	Right Mousebutton, down		- change light color
 */

#include <aux.h>
#include <math.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <stdlib.h>

/*  Function Prototypes  */

GLvoid initialize( char * );
GLvoid drawScene( GLvoid );
GLvoid perspectiveViewing( GLfloat aspect );
GLvoid resize( GLsizei, GLsizei );
GLvoid initLighting( GLvoid );
void   printHelp( char *progname );

GLvoid makeKooshList( GLvoid );
GLvoid drawKoosh(GLvoid);

GLvoid modifyMaterial( AUX_EVENTREC * );
GLvoid modifyLightColor( AUX_EVENTREC * );
GLvoid modifyLModel( AUX_EVENTREC * );

GLuint kooshIndex, properties[3];

#define MATERIAL	0
#define LIGHT		1
#define LMODEL		2

main ( int argc, char *argv[])
{
	initialize( argv[0] );

	auxMainLoop( drawScene );
}

GLvoid
initialize( char *progname )
{
	GLint winSize, width, height;

	auxGetScreenSize( &width, &height );
	winSize = ( width < height ) ? width : height;

	auxInitPosition( winSize / 4, winSize / 4, winSize / 2,
	    winSize / 2 );
	auxInitDisplayMode( AUX_RGBA | AUX_DOUBLE | AUX_DEPTH);
	auxInitWindow("koosh     MOUSE BUTTONS mod light props" );

	auxAnimation( AUX_ON );

	auxReshapeFunc( resize );

	auxKeyFunc( AUX_ESCAPE, auxQuit );

	auxMouseFunc( AUX_LEFTBUTTON, AUX_MOUSEDOWN, 
		modifyMaterial );
	auxMouseFunc( AUX_MIDDLEBUTTON, AUX_MOUSEDOWN, 
		modifyLModel );
	auxMouseFunc( AUX_RIGHTBUTTON, AUX_MOUSEDOWN, 
		modifyLightColor );

	glClearColor( 0, 0, 0, 1 );

	perspectiveViewing( 1.0 );

	glEnable(GL_DEPTH_TEST);

	initLighting();

	printHelp( progname );

	makeKooshList();
}

void
printHelp( char *progname )
{
	fprintf( stdout, 
	"\n%s draws a koosh ball.  When various mouse buttons are \n"\
	"pressed, the properties of the koosh ball and lighting \n"\
	"are modified.  The properties of the koosh ball are kept \n"
	"in display lists which are in turn called by the kooshball\n"
	"display list.\n"\
	"Escape key			- exit the program\n"\
	"Left Mousebutton, down		- change diffuse material color\n"\
	"Middle Mousebutton, down 	- toggle light model\n"\
	"Right Mousebutton, down	- change light color\n" ,
	progname );
}


GLfloat mat_ambient[] = { 0.25, 0.05, 0.4, 1.0 };
GLfloat mat_diffuse[] = { 0.5, 0.1, 0.8, 1.0 };
GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
GLfloat mat_shininess[] = { 10.0 };

GLfloat light_ambient[] = { 0.75, 0.95, 0.6, 1.0 };
GLfloat light_diffuse[] = { 0.5, 0.9, 0.2, 1.0 };
GLfloat light_specular[] = { 0.0, 0.0, 0.0, 1.0 };
GLfloat light_position[] = { 1.0, 1.0, 1.0, 0.0 };

GLfloat lmodel_ambient[] = { 0.2, 0.2, 0.2, 1.0 };
GLfloat lmodel_viewer[] = { 0.0 };

GLvoid 
initLighting( GLvoid )
{

	properties[MATERIAL] = glGenLists( 3 );
	glNewList( properties[MATERIAL], GL_COMPILE );
		glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
		glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
		glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
		glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
	glEndList();

	properties[LIGHT] = properties[MATERIAL] + 1;
	glNewList( properties[LIGHT], GL_COMPILE );
		glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
		glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
		glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);
	glEndList();

	properties[LMODEL] = properties[MATERIAL] + 2;
	glNewList( properties[LMODEL], GL_COMPILE );
		glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
		glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER, 
				lmodel_viewer);
	glEndList();

	glLightfv(GL_LIGHT0, GL_POSITION, light_position);

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
}

GLvoid
modifyMaterial( AUX_EVENTREC *event )
{
	int i;

	printf("Modifying MATERIAL properties\n");

	for (i = 0; i < 3; i++) {
		mat_ambient[i] = drand48();
		mat_diffuse[i] = drand48();
		mat_specular[i] = drand48();
	}
	mat_shininess[0] = drand48() * 128.0;

	glNewList( properties[MATERIAL], GL_COMPILE );
		glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
		glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
		glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
		glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
	glEndList();
}

GLvoid
modifyLightColor( AUX_EVENTREC *event )
{
	int i;

	printf("Modifying LIGHT properties\n");

	for (i = 0; i < 3; i++) {
		light_ambient[i] = drand48();
		light_diffuse[i] = drand48();
		light_specular[i] = drand48();
	}

	glNewList( properties[LIGHT], GL_COMPILE );
		glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
		glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
		glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);
	glEndList();
}

GLvoid
modifyLModel( AUX_EVENTREC *event )
{
	int i;

	printf("Modifying LIGHT MODEL\n");

	for (i = 0; i < 3; i++) {
		lmodel_ambient[i] = drand48();
	}
	lmodel_viewer[0] = !lmodel_viewer[0];

	glNewList( properties[LMODEL], GL_COMPILE );
		glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
		glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER, 
				lmodel_viewer);
	glEndList();
}

GLvoid
makeKooshList( GLvoid )
{

	kooshIndex = glGenLists(1);
	glNewList( kooshIndex, GL_COMPILE );
		glCallLists( 3, GL_UNSIGNED_INT, properties );
		drawKoosh();
	glEndList();
}

GLvoid
drawKoosh( GLvoid )
{
#define MAX_STRANDS 60
#define RADIAN_INCREMENT 2*M_PI/MAX_STRANDS
#define DEGREE_INCREMENT 360.0/MAX_STRANDS

	register int i, j;
	static GLfloat origin[] = { 0.0, 0.0 };
	static GLfloat v[] = { 0.0, 0.0, 0.0 };
	float radians;

	for( i = 0; i < MAX_STRANDS; i++ )
	{
		glRotatef( i * DEGREE_INCREMENT, 0, 1, 0 );

		glBegin( GL_LINES );
		for( j = 0; j < MAX_STRANDS; j++ )
		{
			radians = j * RADIAN_INCREMENT;
			v[0] = cosf( radians );
			v[1] = sinf( radians );

			glNormal3fv( v );
			glVertex2fv( origin );
			glVertex2fv( v );
		}
		glEnd();
	}
}

GLvoid
resize( GLsizei width, GLsizei height )
{
	GLfloat     aspect;
	glViewport( 0, 0, width, height );

	aspect = (GLfloat) (width) / height;
	perspectiveViewing( aspect );
}


GLvoid
perspectiveViewing( GLfloat aspect )
{
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0, aspect, 1.0, 9.0 );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	glTranslatef( 0.0, 0.0, -5.0 );
}

GLvoid
drawScene( GLvoid )
{
	static GLfloat spin;

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	glPushMatrix();
		glRotatef( spin, 0, 1, 0 );
		glCallList( kooshIndex );
	glPopMatrix();

	auxSwapBuffers();

	spin = fmodf( spin + 5.0, 360.0 );
}
