/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */
/*
 *  fly.c This program allows the user to "fly" through a 
 *	simple virtual world. 
 *
 * Utility routines are located in flyutil.c
 *
 * MODIFYING THE PROGRAM
 * ---------------------
 * New shapes should be added to the world in the routine
 * drawShapes(). 
 *
 * If new shapes are to be on the path of the autopilot,
 * they must be near a circle centered at the world 
 * origin with a radius of 10.0, parallel to and
 * raised above the 'floor,' the X-Z plane.
 *
 * As a starting point, here are some good places to center your
 * objects:
 *		( 0.0, 1.0, 10.0 )
 *		( 7.0, 1.0, 7.0  )
 *		( 10.0, 1.0, 0.0 )
 *		( 7.0, 1.0, -7.0 )
 *		( 0.0, 1.0, -10.0)
 *		( -7.0, 1.0, -7.0)
 *		( -10.0, 1.0, 0.0)
 *		( -7.0, 1.0, 7.0 )
 *
 *  Copyright 1991, 1992, 1993, 1994, Silicon Graphics,  Inc.
 *  Technical Education Development
 */


/* fly.h contains function prototypes, defines constants, and */
/* includes GL/gl.h, GL/glu.h, math.h, stdio.h, and aux.h */
#include "fly.h"  

/* Global Variables */

static enum shapes { GRID, TETRA, CUBE, OCTA, DODECA, ICOSA, CONE, 
	CYLINDER, SPHERE, TORUS };
static enum drawModes { WIREFRAME, SOLID, SIMPLE };

GLsizei     winWidth, winHeight;
GLboolean moving = GL_FALSE;
GLboolean autopilot = GL_FALSE;		/* is autopilot mode on? */
GLboolean debugFlag = GL_FALSE;		/* print eye position info? */

/* variables declared in flyutil.c */
extern GLint drawStyle;
extern float ex, ey, ez;	/* eye position */
extern float yaw;		/* eye rotation about y axis in degrees */
extern float pitch;		/* eye rotation about x axis in degrees */		
extern float velocity;		/* eye velocity */

GLvoid
main()
{
	initialize();
	auxMainLoop( drawScene );
}

GLvoid
printHelp( GLvoid )
{
	fprintf(stdout, 
"\nfly.c - This program allows the user to fly through a \n" 
"simple virtual world.\n"
"<a> key	- <a>utopilot start/stop \n"
"<d> key	- <d>ebug mode, prints coords \n"
"<n> key	- <n>ormals drawing toggle \n"
"<r> key	- <r>eset eye position \n"
"SPACE key	- cycle wireframe/flat shaded/smooth \n"
"LEFT Mouse 	- control moving the eye forward \n"
"MIDDLE Mouse 	- control moving the eye backward \n"
"Mouse pointer	- change direction: pull back to go up, forward to dive\n"
"Escape key	- exit the program \n"
"<?> key 	- help, prints this info \n\n" );
}


GLvoid
initialize( GLvoid )
{
	GLfloat aspect;
	GLsizei screenWidth, screenHeight, winSize;
	
	auxGetScreenSize( &screenWidth, &screenHeight );
	winSize = ( screenWidth < screenHeight ) ? 
		screenWidth : screenHeight;
	winWidth = screenWidth / 2; 
	winHeight = screenHeight / 2;
	auxInitPosition( screenWidth / 4, screenHeight / 4, 
		winWidth, winHeight );
	auxInitDisplayMode( AUX_RGBA | AUX_DEPTH | AUX_DOUBLE );
	auxInitWindow( "fly    `a' `n' `r' `SPACE' keys + MIDDDLE/LEFT MOUSE" );

	/* all of the following input callback routines are 
	 * in flyutil.c  */
	auxReshapeFunc( resize );	auxKeyFunc( AUX_ESCAPE, auxQuit );
	
	auxKeyFunc( AUX_SPACE, setDrawStyle );
	auxKeyFunc( AUX_n, toggleDrawNormals );
	auxKeyFunc( AUX_a, setAutopilot );
	auxKeyFunc( AUX_r, initializeEye );
	auxKeyFunc( AUX_d, setDebug );
	auxKeyFunc( AUX_QUESTION, printHelp );
	auxKeyFunc( AUX_SLASH, printHelp );

	/* Set up functions using the mouse */
	auxMouseFunc( AUX_LEFTBUTTON, AUX_MOUSEDOWN, startMove );
	auxMouseFunc( AUX_LEFTBUTTON, AUX_MOUSEUP, stopMove );
	auxMouseFunc( AUX_MIDDLEBUTTON, AUX_MOUSEDOWN, startMoveBack );
	auxMouseFunc( AUX_MIDDLEBUTTON, AUX_MOUSEUP, stopMoveBack );

	auxAnimation( AUX_ON ); 

	glClearColor( 0.0, 0.0, 0.0, 1.0 );
	glClearDepth( 1.0 );

	glEnable( GL_DEPTH_TEST );

	glMatrixMode( GL_PROJECTION );
	aspect = (GLfloat) winWidth / winHeight;
	gluPerspective( 45.0, aspect, 1.0, 50.0 );
	glMatrixMode( GL_MODELVIEW );

	initLighting();
	initializeEye(); /* function in flyutil.c */
	printHelp();
}

/*  Initialize material property, light source, and lighting model, 
 */
GLvoid
initLighting( GLvoid )
{
	GLfloat mat_ambient[] = { 0.25, 0.05, 0.4, 1.0 };
	GLfloat mat_diffuse[] = { 0.5, 0.1, 0.8, 1.0 };

	/* mat_specular and mat_shininess are NOT default values */
	GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
	GLfloat mat_shininess[] = { 10.0 };

	/* light_position is NOT default value */
	GLfloat light_position[] = { 1.0, 1.0, 1.0, 0.0 };
	GLfloat lmodel_ambient[] = { 0.2, 0.2, 0.2, 1.0 };

	glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
	glShadeModel( GL_SMOOTH );
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
}

GLvoid
drawScene( GLvoid )
{
	moveEye();

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	glPushMatrix();

		/* accomplish a viewing transformation by moving
		 * the world relative to the eye
		 */
		glRotatef( pitch, 1.0, 0.0, 0.0 );
		glRotatef( yaw, 0.0, 1.0, 0.0 );
		glTranslatef(-ex,-ey,-ez);
	
		glPushMatrix();
			computeGrid(); /* function in flyutil.c */	
			drawGrid(); /* function in flyutil.c */
		glPopMatrix();
		
		drawShapes();

	glPopMatrix();

	glFlush();
	auxSwapBuffers();
}

GLvoid
drawShapes( GLvoid )
{
	GLfloat shapesDiffuse[] = { 0.5, 0.1, 0.8, 1.0 };

	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, 
		shapesDiffuse);

	drawMovedShape( -5.0, 1.0, 8.7, TETRA ); 
	drawMovedShape( -8.7, 1.0, 0.5, CUBE );
	drawMovedShape( 5.0, 1.0, 8.7, OCTA );
	drawMovedShape( -8.7, 1.0, -5.0, DODECA );
	drawMovedShape( -5.0,1.0,-8.7, ICOSA );
	drawMovedShape( 8.7,1.0,5.0, CONE );
	drawMovedShape( 10.0,1.5,0.0, CYLINDER );
	drawMovedShape( 7.0, 2.0, -7.0, SPHERE );
	drawMovedShape( 0.0, 2.0, -10.0, TORUS );
}

GLvoid
drawMovedShape( GLfloat x, GLfloat y, GLfloat z, GLint currentShape )
{
	glPushMatrix();
		glTranslatef(x, y, z);
		if ( drawStyle == WIRE )
		{
			drawShape( currentShape, WIREFRAME );
		}
		else
		{
			drawShape( currentShape, SOLID );
		}
	glPopMatrix();
}

GLvoid
drawShape( GLint currentShape, GLint drawMode )
{
	switch( currentShape )
	{
		case TETRA:
			drawTetra( drawMode );
			break;
		case CUBE:
			drawCube( drawMode );
			break;
		case OCTA:
			drawOcta( drawMode );
			break;
		case DODECA:
			drawDodeca( drawMode );
			break;
		case ICOSA:
			drawIcosa( drawMode );
			break;
		case CONE:
			drawCone( drawMode );
			break;
		case CYLINDER:
			drawCylinder( drawMode );
			break;
		case SPHERE:
			drawSphere( drawMode );
			break;
		case TORUS:
			drawTorus( drawMode );
			break;
		default:
			break;
	}
}
		
		
GLvoid
drawTetra( GLint drawMode )
{			
	switch( drawMode )
	{
		case WIREFRAME:
			auxWireTetrahedron( 1.0 );
			break;
		case SOLID:
			auxSolidTetrahedron( 1.0 );
			break;
		default:
			break;
	}
}					

GLvoid
drawCube( GLint drawMode )
{			
	switch( drawMode )
	{
		case WIREFRAME:
			auxWireCube( 1.0 );
			break;
		case SOLID:
			auxSolidCube( 1.0 );
			break;
		default:
			break;
	}
}					

GLvoid
drawOcta( GLint drawMode )
{			
	switch( drawMode )
	{
		case WIREFRAME:
			auxWireOctahedron( 1.0 );
			break;
		case SOLID:
			auxSolidOctahedron( 1.0 );
			break;
		default:
			break;
	}
}					


GLvoid
drawDodeca( GLint drawMode )
{			
	switch( drawMode )
	{
		case WIREFRAME:
			auxWireDodecahedron( 1.0 );
			break;
		case SOLID:
			auxSolidDodecahedron( 1.0 );
			break;
		default:
			break;
	}
}					

GLvoid
drawIcosa( GLint drawMode )
{			
	switch( drawMode )
	{
		case WIREFRAME:
			auxWireIcosahedron( 1.0 );
			break;
		case SOLID:
			auxSolidIcosahedron( 1.0 );
			break;
		default:
			break;
	}
}					

GLvoid
drawCone( GLint drawMode )
{			
	glRotatef(-90.0, 1.0, 0.0, 0.0);
	switch( drawMode )
	{
		case WIREFRAME:
			auxWireCone( 1.0, 2.0 );
			break;
		case SOLID:
			auxSolidCone( 1.0, 2.0 );
			break;
		default:
			break;
	}
}					

GLvoid
drawCylinder( GLint drawMode )
{			
	glRotatef(90.0, 1.0, 0.0, 0.0);
	switch( drawMode )
	{
		case WIREFRAME:
			auxWireCylinder( 1.0, 2.0 );
			break;
		case SOLID:
			auxSolidCylinder( 1.0, 2.0 );
			break;
		default:
			break;
	}
}					

GLvoid
drawSphere( GLint drawMode )
{			
	switch( drawMode )
	{
		case WIREFRAME:
			auxWireSphere( 1.0 );
			break;
		case SOLID:
			auxSolidSphere( 1.0 );
			break;
		default:
			break;
	}
}					

GLvoid
drawTorus( GLint drawMode )
{			
	static GLint spinTorus = 0;
	
	glRotatef( (GLfloat) spinTorus, 0.0, 1.0, 0.0 );
	switch( drawMode )
	{
		case WIREFRAME:
			auxWireTorus( 0.25, 0.75 );
			break;
		case SOLID:
			auxSolidTorus( 0.25, 0.75 );
			break;
		default:
			break;
	}
	/* increase spinTorus, taking modulus so angle won't
	 * exceed 360 degrees
	 */
	spinTorus = ( spinTorus + (int)(20*velocity) ) % 360;
}

