/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/*				fog.c
 * This program demonstrates using different types of fog. 
 * Pressing the left mouse button chooses between 3 types of fog:
 * exponential, exponential squared, and linear.
 * Pressing the up and down arrow keys adjusts the density value.
 * The start and end values for the linear fog are fixed.
 *
 *		Escape key			- exit the program
 *		UP Arrow Key			- increase the fog density
 *		DOWN Arrow Key			- decrease the fog density
 *		LEFT Mousebutton, down		- change the fog blend function
 */

#include <aux.h>
#include <math.h>
#include <stdio.h>

/* Function Prototypes */

GLvoid initialize();
GLvoid drawScene( GLvoid );
GLvoid resize( GLsizei width, GLsizei height );
GLvoid printHelp( GLvoid );
GLvoid initLighting( GLvoid );
GLvoid cycleFog( AUX_EVENTREC *event );
GLvoid incDensity( GLvoid );
GLvoid decDensity( GLvoid );

/* Global Variables */

GLint		fogMode;
GLfloat		fogDensity;

main ()
{
	initialize();

	auxMainLoop( drawScene );
}

GLvoid
initialize( )
{
	GLsizei width, height, winSize;
	GLfloat fogColor[4] = {0.5, 0.5, 0.5, 1.0};

	auxGetScreenSize( &width, &height );
	winSize = ( width < height ) ? width : height;
	auxInitPosition(winSize/4, winSize/4, winSize/2, winSize/2);
	auxInitDisplayMode( AUX_RGBA | AUX_DEPTH | AUX_DOUBLE );
	auxInitWindow( "fog    UP/DOWN arrow keys LEFTMOUSE" );
	
	auxReshapeFunc( resize );
	auxAnimation( AUX_ON );
	
	auxMouseFunc( AUX_LEFTBUTTON, AUX_MOUSEDOWN, cycleFog );

	auxKeyFunc( AUX_ESCAPE, auxQuit );
	auxKeyFunc( AUX_UP, incDensity );
	auxKeyFunc( AUX_DOWN, decDensity );
	
	glEnable( GL_DEPTH_TEST );
	glMatrixMode( GL_PROJECTION );
	gluPerspective( 45.0, 1.0, 3.0, 13.0 );
	glMatrixMode( GL_MODELVIEW );
	glTranslatef( 0.0, 0.0, -8.0 );

	initLighting(); 

	/* clear background to the same color as the fog */
	glClearColor(0.5, 0.5, 0.5, 1.0);

	fogMode = GL_EXP;
	fogDensity = 0.25;

	/* setup and enable fog */
	glFogi( GL_FOG_MODE, fogMode );
	glFogfv( GL_FOG_COLOR, fogColor );
	glFogf( GL_FOG_DENSITY, fogDensity );
	glHint( GL_FOG_HINT, GL_DONT_CARE );
	glEnable( GL_FOG );

	printHelp();
	printf("Fog mode is GL_EXP, density = %4.2f\n", fogDensity);
}

/* Initialize material property, light source, lighting model, 
 * and depth buffer.
 */
GLvoid
initLighting( GLvoid )
{

	/* mat_specular and mat_shininess are NOT default values */
	GLfloat mat_specular[] = { 0.8, 0.8, 0.8, 1.0 };
	GLfloat mat_shininess[] = { 10.0 };

	/* light_position is NOT default value */
	GLfloat light_position[] = { 1.0, 1.0, 1.0, 0.0 };

	GLfloat lmodel_ambient[] = { 0.2, 0.2, 0.2, 1.0 };

	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
	glShadeModel( GL_SMOOTH );
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
}

GLvoid
printHelp( GLvoid )
{
	fprintf(stdout,"\nfog\n\n"\
		"Escape key			- exit the program\n"\
		"UP Arrow Key			- increase the fog density\n"\
		"DOWN Arrow Key			- decrease the fog density\n"\
		"LEFT Mousebutton, down		- change fog blend function\n"
	);
}

GLvoid
cycleFog (AUX_EVENTREC *event)
{
	if (fogMode == GL_EXP) {
		fogMode = GL_EXP2;
		printf ("Fog mode is GL_EXP2\n");
	}
	else if (fogMode == GL_EXP2) {
		fogMode = GL_LINEAR;
		glFogf (GL_FOG_START, 5.0);
		glFogf (GL_FOG_END, 11.0);
		printf ("Fog mode is GL_LINEAR: start = 5.0, end = 11.0\n");
	}
	else if (fogMode == GL_LINEAR) {
		fogMode = GL_EXP;
		printf ("Fog mode is GL_EXP\n");
	}
	glFogi (GL_FOG_MODE, fogMode);
}

GLvoid
decDensity( GLvoid )
{
	fogDensity -= 0.05;
	if (fogDensity < 0.0)
	fogDensity = 0.0;

	glFogf (GL_FOG_DENSITY, fogDensity);
	printf ("Fog density = %4.2f\n", fogDensity);
}

GLvoid
incDensity( GLvoid )
{
	fogDensity += 0.05;
	if (fogDensity > 1.0)
	fogDensity = 1.0;

	glFogf (GL_FOG_DENSITY, fogDensity);
	printf ("Fog density = %4.2f\n", fogDensity);
}

GLvoid
resize( GLsizei width, GLsizei height )
{
	GLfloat    aspect;

	glViewport( 0, 0, width, height );

	aspect = (GLfloat) width / height;

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0, aspect, 3.0, 13.0 );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	glTranslatef( 0.0, 0.0, -8.0 ); 
}

GLvoid
drawScene( GLvoid )
{
	static GLfloat angle = 80.0;
	int i,  slices = 8;
	
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	glPushMatrix(); 
	
	glRotatef( angle, 0.0, 1.0, 0.0 );
	angle += 0.5;
	if (angle > 360.0)
		angle = 0.0;

	for ( i = 0; i < slices; i++ )
	{
		glColor3f( i/10.0, i/10.0, 1.0 - i/10.0 ); 
		glPushMatrix(); 
		glRotatef( i * 360.0/slices, 0, 0, 1 );
		glTranslatef( 1.5, 0.0, 0.0 );
		glRotatef( i * 360.0/slices, 0, 1, 0 );
		auxSolidTorus( 0.25, 0.75 );
		glPopMatrix();
	}
	glPopMatrix();

	auxSwapBuffers();
}
