/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/* comboTrim_dlist.c
 * This program draws a rotating NURBS surface with a half-oval with
 * a triangular base trimmed from its center.
 *
 * For versions of the NURBS surface are create.  Three are in 
 * display lists which differ in the manner in which they are 
 * filled.  The fourth is not put in a display list.
 * 
 * Automatic load matrix is turned off for the display lists for speed.
 * Note the difference in tessellation when in outline polygon mode.
 *
 *	Escape key		- exit the program
 *	SPACE key		- toggle display list/immediate mode
 *	Left Mousebutton, down	- filled/wireframe/patch mode
 */
#include <aux.h>
#include <math.h>
#include <stdio.h>

/*  Function Prototypes  */
GLvoid initialize();
GLvoid drawScene( GLvoid );
GLvoid resize( GLsizei, GLsizei );
GLvoid initLighting( GLvoid );
GLvoid initSurface( GLvoid );
GLuint makeList( GLUnurbsObj *, GLuint );
GLvoid drawSurface( GLUnurbsObj * );
GLvoid toggleDisplayLists( GLvoid );
GLvoid toggleFillMode( AUX_EVENTREC * );

void printNurbsError( GLenum );
void printHelp( char * );

GLfloat ctlpoints[4][4][3];

/* NURBS objects */
GLUnurbsObj *immediateNurb, *filledNurb, *outlineNurb, *patchNurb;

/* Display list indices */
GLuint  surfaceIndex, filledIndex, outlineIndex, patchIndex;

/* Matrices used when auto load sampling is turned off */
GLfloat modelView[16], projection[16]; 
GLint   viewport[4];

static GLboolean displayLists = 1;

void
main(int argc, char *argv[])
{
	initialize();
	printHelp(argv[0]);
	auxMainLoop( drawScene );
}

void
printHelp( char *progname )
{
	fprintf(stdout, "\n%s\n\n"\
		"Escape key		- exit the program\n"\
		"SPACE key		- toggle display list/immediate mode\n"\
		"Left Mousebutton, down	- filled/wireframe/patch mode\n\n", 
		progname);
}

GLvoid
resize( GLsizei width, GLsizei height )
{
	GLfloat    aspect;

	glViewport( 0, 0, width, height );

	aspect = (GLfloat) width / height;

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0, aspect, 3.0, 20.0 );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	glTranslatef( 0.0, 0.0, -15.0 ); 
}

/*  Initialize material property, light source, lighting model, 
 *  and depth buffer.
 */
GLvoid
initLighting( GLvoid )
{
	GLfloat mat_diffuse[] = { 0.8, 0.8, 0.2, 1.0 };
	GLfloat mat_specular[] = { 0.8, 0.0, 0.8, 1.0 };
	GLfloat mat_emission[] = { 0.0, 0.0, 0.0, 1.0 };
	GLfloat mat_shininess[] = { 30.0 };

	GLfloat light_position[] = { 1.0, 0.0, 1.0, 0.0 };

	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, 
		mat_diffuse);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, mat_emission);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, mat_shininess);
	glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, 1.0);
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);

	glShadeModel( GL_SMOOTH );
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
}


GLvoid
initialize( )
{
	GLsizei  width, height, winSize;

	auxGetScreenSize( &width, &height );
	winSize = ( width < height ) ? width : height;
	auxInitPosition( winSize / 4, winSize / 4, winSize / 2, 
		winSize / 2 );
	auxInitDisplayMode( AUX_RGBA | AUX_DEPTH | AUX_DOUBLE );
	auxInitWindow( "comboTrim_dlist" );
	auxAnimation( AUX_ON );

	auxReshapeFunc( resize );
	auxKeyFunc( AUX_ESCAPE, auxQuit );
	auxKeyFunc( AUX_SPACE, toggleDisplayLists );

	auxMouseFunc( AUX_LEFTBUTTON, AUX_MOUSEDOWN, toggleFillMode );

	glClearColor( 0, 0, 0, 1 );
	glEnable( GL_DEPTH_TEST );

	glMatrixMode( GL_PROJECTION );
	gluPerspective( 45.0, 1.0, 3.0, 20.0 );
	glMatrixMode( GL_MODELVIEW );
	glTranslatef( 0.0, 0.0, -15.0 ); 

	initLighting(); 

	initSurface();

	glEnable( GL_AUTO_NORMAL );

	/* Save the current matrices to be loaded with the
	 * display listed surfaces.  The surfaces will all be 
	 * tessellated using these matrices 
	 */
	glGetFloatv(GL_MODELVIEW_MATRIX, modelView);
	glGetFloatv(GL_PROJECTION_MATRIX, projection);
	glGetIntegerv(GL_VIEWPORT, viewport);

	/* Save three versions of the NURBS surface in display lists:
	 * one for each of the various fill modes:
	 *	GLU_FILL, GLU_OUTLINE_POLYGON, GLU_OUTLINE_PATCH
	 */
	outlineNurb = gluNewNurbsRenderer();
	outlineIndex = makeList(outlineNurb, GLU_OUTLINE_POLYGON); 

	filledNurb = gluNewNurbsRenderer();
	filledIndex = makeList(filledNurb, GLU_FILL);

	patchNurb = gluNewNurbsRenderer();
	patchIndex = makeList(patchNurb, GLU_OUTLINE_PATCH); 

	/* Create a NURBS surface for when we are in immediate mode */
	immediateNurb = gluNewNurbsRenderer();
	gluNurbsProperty( immediateNurb, GLU_DISPLAY_MODE, GLU_OUTLINE_POLYGON);

	surfaceIndex = outlineIndex;

	/* Set up callback function for error messages
	 * must set one up for each NURBS object
	 */
	gluNurbsCallback( outlineNurb, GLU_ERROR, printNurbsError);
	gluNurbsCallback( filledNurb, GLU_ERROR, printNurbsError);
	gluNurbsCallback( patchNurb, GLU_ERROR, printNurbsError);
	gluNurbsCallback( immediateNurb, GLU_ERROR, printNurbsError);

	fprintf( stdout, "Display List Mode is on\n" );
	fprintf( stdout, "fillMode = GLU_OUTLINE_POLYGON\n" );
}

/* Create a display list for the NURBS object using
 * the specified fill mode.  We turn off auto load sampling
 * to take full advantage of display lists.  Consequently, we
 * need to load a fixed set of matrices to be used for 
 * tessellation and culling.
 *
 * Returns the index of the newly created display list.
 */
GLuint
makeList( GLUnurbsObj *theNurb, GLuint fillMode )
{
	GLuint newIndex;

	/* Set display mode and tolerance for this object */
	gluNurbsProperty(theNurb, GLU_DISPLAY_MODE, fillMode);
	gluNurbsProperty(theNurb, GLU_SAMPLING_TOLERANCE, 50.0);

	/* Turn off automatic loading of matrices and set up 
	 * sampling matrices to be used when display list is 
	 * created
	 */
	gluNurbsProperty(theNurb, GLU_AUTO_LOAD_MATRIX, GL_FALSE);
	gluLoadSamplingMatrices(theNurb, modelView, projection,
				viewport);

	/* Create a new surface of the appropriate fill type */
	newIndex = glGenLists( 1 );
	glNewList( newIndex, GL_COMPILE );
		drawSurface( theNurb );
	glEndList();

	return newIndex;
}

/* Define user input functions for mouse events */
GLvoid  
toggleDisplayLists( GLvoid )
{

	displayLists = !displayLists;
	printf("Display List Mode is %s\n", 
		(displayLists ? "ON" : "OFF"));
}

GLvoid  
toggleFillMode( AUX_EVENTREC *event )
{
	static GLuint fillMode = GLU_OUTLINE_POLYGON;

	switch (fillMode) {
	case GLU_OUTLINE_POLYGON:
		surfaceIndex = filledIndex;
		fillMode = GLU_FILL;
		printf("fillMode is GLU_FILL\n"); 
		break;
	case GLU_FILL:
		surfaceIndex = patchIndex;
		fillMode = GLU_OUTLINE_PATCH;
		printf("fillMode is GLU_OUTLINE_PATCH\n"); 
		break;
	case GLU_OUTLINE_PATCH:
		surfaceIndex = outlineIndex;
		fillMode = GLU_OUTLINE_POLYGON;
		printf("fillMode is GLU_OUTLINE_POLYGON\n"); 
		break;
	}

	/* Change the immediate fill mode so that it will 
	 * match that of the display listed surface
	 */
	gluNurbsProperty(immediateNurb, GLU_DISPLAY_MODE, fillMode);
}


/*  Initializes the control points of the surface to a small hill.
 *  The control points range from -3 to +3 in x, y, and z
 */
GLvoid 
initSurface(void)
{
	int s, t;

	for (s = 0; s < 4; s++) {
		for (t = 0; t < 4; t++) {
			ctlpoints[s][t][0] = 2.0*((GLfloat)s - 1.5);
			ctlpoints[s][t][1] = 2.0*((GLfloat)t - 1.5);

			if ( (s == 1 || s == 2) && (t == 1 || t == 2))
				ctlpoints[s][t][2] = 3.0;
			else
				ctlpoints[s][t][2] = -3.0;
		}
	}				
}

/* Draw a NURBS surface with a half-oval cut out of it */
GLvoid
drawSurface( GLUnurbsObj *theNurb )
{
	GLfloat knots[8] = {0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0};
	GLfloat edgePt[5][2] = /* counter clockwise */
	   {{0.0, 0.0}, {1.0, 0.0}, {1.0, 1.0}, {0.0, 1.0}, {0.0, 0.0}};

	GLfloat curveKnots[8] = 
	   {0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0};
	GLfloat curvePt[4][2] = /* clockwise */ 
	   {{0.1, 0.5}, {0.1, 0.9}, {0.9, 0.9}, {0.9, 0.5}};
	GLfloat pwlPt[4][2] = /* clockwise */ 
	   {{0.9, 0.5}, {0.5, 0.1}, {0.1, 0.5}};

	gluBeginSurface(theNurb);
		gluNurbsSurface(theNurb, 
			8, knots,
			8, knots,
			4 * 3, 3,
			&ctlpoints[0][0][0], 
			4, 4,
			GL_MAP2_VERTEX_3);
		gluBeginTrim (theNurb);
		    gluPwlCurve (theNurb, 5, &edgePt[0][0], 2,
				GLU_MAP1_TRIM_2);
		gluEndTrim (theNurb);
		gluBeginTrim (theNurb);
		    gluNurbsCurve (theNurb, 8, curveKnots, 2, 
			    &curvePt[0][0], 4, GLU_MAP1_TRIM_2);
		    gluPwlCurve (theNurb, 3, &pwlPt[0][0], 2, 
				GLU_MAP1_TRIM_2);
		gluEndTrim (theNurb);
	gluEndSurface(theNurb);
}

/* Use the NURBS error number passed in to get a meaningful
 * string describing the error.
 */
void
printNurbsError( GLenum errnum )
{
	printf("NURBS error: %s\n", gluErrorString( errnum ) );
}

GLvoid
drawScene( GLvoid )
{
	static GLfloat angle = 0.0;

	
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	glPushMatrix(); 

		glRotatef( angle, 0.0, 1.0, 0.0 );

		if (displayLists) {
			/* invoke display list to create surface */
			glCallList( surfaceIndex );
		} else {
			drawSurface( immediateNurb );
		}

	glPopMatrix();

	angle += 5.0;
	if (angle > 360.0)
		angle = 0.0;

	auxSwapBuffers();
}
