/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/*		selectSolar.c
 * Selection is demonstrated here. A simple solar system
 * is drawn first in render mode, then in select mode with
 * a narrower perspective, selecting the center of the scene.
 * Names are assigned to the shapes, and if they score hits
 * within the selected region the selected names can be retrieved. 
 */
#include <stdio.h>
#include <math.h>
#include "aux.h"

/* Function Prototypes */

GLvoid printHelp( char * );
GLvoid initialize( char * );
GLvoid drawScene( GLvoid );
GLvoid drawSolar( GLboolean );
GLvoid resize( GLsizei, GLsizei );
GLvoid initLighting( GLvoid );
GLvoid toggleSelection( GLvoid );
GLvoid drawViewVolume( GLdouble, GLdouble, GLdouble, GLdouble );
GLvoid printHits( GLint, GLuint [] );

GLfloat aspect;

GLboolean selectionFlag = GL_TRUE;

/* objects to be drawn */
enum shapes { SUN, EARTH, MOON };
char *shapeNames[] =  { "Sun", "Earth", "Moon" };

/* size of selection buffer */
#define BUFSIZE 512

void
main( int argc, char **argv )
{
	initialize( argv[0] );
	auxMainLoop( drawScene );
}

GLvoid
printHelp( char *progname )
{
	fprintf(stdout, "\n%s\nThis program demonstrates selection\n"
		"SPACE key			- toggle selection mode \n"
		"Escape key			- exit the program \n\n",
		progname );
}

GLvoid
resize( GLsizei width, GLsizei height )
{
	GLfloat    aspect;

	glViewport( 0, 0, width, height );

	aspect = (GLfloat)width / (GLfloat)height;

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0, aspect, 6.0, 14.0 );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
}

GLvoid toggleSelection( GLvoid )
{
	selectionFlag = !selectionFlag;    
}

/* Initialize lighting
 */
GLvoid
initLighting( GLvoid )
{
	GLfloat mat_specular[] = { 0.8, 0.8, 0.8, 1.0 };
	GLfloat mat_shininess[] = { 10.0 };
	GLfloat mat_ambient[] = { 0.0, 0.0, 0.0, 1.0 };

	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
	glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
	glEnable( GL_LIGHTING );
	glEnable( GL_LIGHT0 );
}

GLvoid
initialize( char *progname )
{
	GLsizei  width, height;

	auxGetScreenSize( &width, &height );
	auxInitPosition(width/4, height/4, width/2, height/2);
	auxInitDisplayMode( AUX_RGBA | AUX_DEPTH | AUX_DOUBLE );
	auxInitWindow( progname );
	
	auxReshapeFunc( resize );
	auxAnimation( AUX_ON );

	auxKeyFunc( AUX_ESCAPE, auxQuit );
	auxKeyFunc( AUX_SPACE, toggleSelection );
	
	glClearColor( 0, 0, 0, 1 );
	glEnable( GL_DEPTH_TEST );

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	aspect = (GLfloat)width / (GLfloat)height;
	gluPerspective( 45.0, aspect, 6.0, 14.0 );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();

	initLighting(); 
	printHelp( progname );
}

GLvoid  
drawScene( GLvoid )
{
	GLuint selectBuf[BUFSIZE];
	GLint hits;

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	/* draw selection viewing volume slightly inside 
	 * actual selection volume so that it is visible 
	 */
	drawViewVolume( 15.0, aspect, 6.01, 13.99 );  
	drawSolar( GL_FALSE );
	if ( selectionFlag ) {
		glSelectBuffer (BUFSIZE, selectBuf);
		(GLvoid) glRenderMode( GL_SELECT );
		glInitNames();
		glPushName( -1 );

		glMatrixMode( GL_PROJECTION );
		glPushMatrix();
			glLoadIdentity();
			/* set fovy much smaller than normal to select 
			 * only center of scene
			 */
			gluPerspective( 15.0, aspect, 6.0, 14.0 );
			glMatrixMode( GL_MODELVIEW );
			drawSolar( GL_TRUE );
	
			hits = glRenderMode( GL_RENDER );
			printHits ( hits, selectBuf );	
			glMatrixMode( GL_PROJECTION );
		glPopMatrix();
		glMatrixMode( GL_MODELVIEW );
	}
}

GLvoid
drawSolar( GLboolean selecting )
{
	static int year = 0;
	static float sun_diffuse[] = { 1.0, 1.0, 0.0, 1.0 };
	static float sun_specular[] = { 1.0, 1.0, 1.0, 1.0 };
	static float no_specular[] = { 0.0, 0.0, 0.0, 1.0 };
	static float earth_diffuse[] = { 0.0, 0.3, 1.0, 1.0 };
	static float moon_diffuse[] = { 1.0, 1.0, 1.0, 1.0 };
	static float position[] = { 0.0, 0.0, 1.0, 0.0 };
	/* simulate local light centered at sun by rotating an
	 * infinite light */
	static float sun_position[] = { -1.0, 0.0, 0.0, 0.0 };

	glPushMatrix();

		/* sun surface lit by light attached to eye, 
		 * thus before viewing */
		glLightfv(GL_LIGHT0, GL_POSITION, position);

		/* accomplish a viewing transformation by moving
		 * the world relative to the eye
		 */
		glTranslatef(0.0, 0.0, -10.0);
		glRotatef( 10.0, 1.0, 0.0, 0.0 );

		glPushMatrix();
			glRotatef (90.0, 1.0, 0.0, 0.0);
			glMaterialfv(GL_FRONT, GL_DIFFUSE, sun_diffuse);
			glMaterialfv(GL_FRONT, GL_SPECULAR, sun_specular);
			glLoadName(SUN);
			auxSolidSphere(0.7);
		glPopMatrix();

		glPushMatrix();
			glRotatef ((float) year, 0.0, 1.0, 0.0);
			glTranslatef (2.5, 0.0, 0.0);
			glPushMatrix();
				/* infinite light looks like local light at 
				 * sun because rotating in sync with planet */
				glLightfv(GL_LIGHT0, GL_POSITION, 
					sun_position);
				glRotatef (90.0, 1.0, 0.0, 0.0);
				glMaterialfv(GL_FRONT, GL_SPECULAR, 
					no_specular);
				glMaterialfv(GL_FRONT, GL_DIFFUSE,
					earth_diffuse);
				glLoadName(EARTH);
				auxSolidSphere(0.5);
			glPopMatrix();

			glPushMatrix();
				glRotatef ((float) ((year * 5) % 360),
					0.0, 1.0, 0.0);
				glTranslatef (1.0, 0.0, 0.0);
				glRotatef (90.0, 1.0, 0.0, 0.0);
				glMaterialfv(GL_FRONT, GL_DIFFUSE,
					moon_diffuse);
				glPushName(MOON);
				auxSolidSphere(0.2);
			glPopMatrix();
		glPopMatrix();

	glPopMatrix();

	if ( selecting ) {
		glFlush();
		year = (year + 1) % 360;
	} else {
		auxSwapBuffers();
	}
}

GLvoid 
printHits (GLint hits, GLuint buffer[] )
{
	unsigned int i, j, nameCount;
	GLuint *ptr;
	static GLdouble depthNormalizeFactor;
	static GLboolean init = GL_FALSE;
	GLint depthBits;  
	GLdouble depthNormalized;
	
	if (!init)
	{
		init = GL_TRUE;
		glGetIntegerv( GL_DEPTH_BITS, &depthBits );
		depthNormalizeFactor = pow(2, depthBits) - 1;
		fprintf( stdout, 
			"\ndepthBits=%d, depthNormalizeFactor=%f\n", 
			depthBits, depthNormalizeFactor); 
	}
	
	printf ("hits = %d\n", hits);
	ptr = (GLuint *) buffer;
	for (i = 0; i < hits; i++) 
	{	/*  for each hit  */
		nameCount = *ptr; ptr++;
		printf( " number of names for this hit = %d\n", 
			nameCount ); 
		depthNormalized = 
			(GLdouble)(*ptr)/depthNormalizeFactor; ptr++;
		printf( " z1 = %f;", depthNormalized );
		depthNormalized = 
			(GLdouble)(*ptr)/depthNormalizeFactor; ptr++;
		printf( " z2 = %f;\n", depthNormalized );
		printf( " names are: " );
		for ( j = 0; j < nameCount; j++, ptr++ ) 
		{	/*  for each name */
			fprintf( stdout, "\t%s", shapeNames[*ptr] );
		}
		printf( "\n\n" );
	}
}

/*
 *  draw a perspective viewing volume indicating selected region
 */
GLvoid 
drawViewVolume ( GLdouble fovy, GLdouble aspect, GLdouble near, GLdouble far )
{
	GLfloat halfFovyRadians;
	GLfloat farLeft, farRight, farBottom, farTop;
	GLfloat nearLeft, nearRight, nearBottom, nearTop;
	static GLint init = 0;
		
	halfFovyRadians = (fovy/2.0) * (M_PI/180.0);
	
	farTop = far * tan( halfFovyRadians );
	farBottom = -farTop;
	farRight = aspect * farTop;
	farLeft = -farRight;

	nearTop = near * tan( halfFovyRadians );
	nearBottom = -nearTop;
	nearRight = aspect * nearTop;
	nearLeft = -nearRight;

	/* offset near clipping plane slightly inwards
	 * so that it will be visible.  
	 */
	nearTop = near * tan( halfFovyRadians ) -.03;
	nearBottom = -nearTop;
	nearRight = aspect * nearTop;
	nearLeft = -nearRight;
	
	glDisable( GL_LIGHTING );
	glColor3f (1.0, 0.0, 0.0);
	/* draw far clipping plane */
	glBegin( GL_LINE_LOOP );
		glVertex3f (farLeft, farBottom, -far);
		glVertex3f (farRight, farBottom, -far);
		glVertex3f (farRight, farTop, -far);
		glVertex3f (farLeft, farTop, -far);
	glEnd();

	glColor3f( 1.0, 1.0, 0.0 );
	/* draw near clipping plane */
	glBegin( GL_LINE_LOOP );
		glVertex3f (nearLeft, nearBottom, -near);
		glVertex3f (nearRight, nearBottom, -near);
		glVertex3f (nearRight, nearTop, -near);
		glVertex3f (nearLeft, nearTop, -near);
	glEnd();

	/* connect clipping planes */    
	glBegin( GL_LINES );					/* 4 lines */
		glVertex3f (nearLeft, nearBottom, -near);
		glVertex3f (farLeft, farBottom, -far);

		glVertex3f (nearRight, nearBottom, -near);
		glVertex3f (farRight, farBottom, -far);

		glVertex3f (nearRight, nearTop, -near);
		glVertex3f (farRight, farTop, -far);

		glVertex3f (nearLeft, nearTop, -near);
		glVertex3f (farLeft, farTop, -far);
	glEnd();
	glEnable( GL_LIGHTING );
}
