/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/* texture.c 
 * This program reads in a texture from a .rgb file, sets up the
 * 	texture and the texture environment, then applies the texture
 * 	to a rectangular polygon using explicit texture coordinates.
 */
 
#include <aux.h>
#include <math.h>
#include <stdio.h>
/* rgbImageFile.h should be found in the ../../include directory */
#include "rgbImageFile.h"

/*  Function Prototypes  */

GLvoid  initialize( int, char ** );
GLvoid  drawScene( GLvoid );
GLvoid 	perspectiveViewing( GLfloat );
GLvoid  resize( GLsizei, GLsizei );
GLvoid  initTexture( unsigned int *, GLsizei, GLsizei );

main ( int argc, char *argv[])
{
	initialize( argc,  argv );

	auxMainLoop( drawScene );
}

GLvoid
initialize( int argc, char *argv[] )
{
	char		*imageFileName = "fish.rgba";
	unsigned int	*image;
	GLsizei		imageWidth, imageHeight;
	GLint		screenWidth, screenHeight;
	GLsizei		windowWidth, windowHeight;
	GLfloat		aspect;

	if (argc < 2) {
		fprintf (stderr, "usage: %s <imageFileName>\n", argv[0] );
	} else
		imageFileName = argv[1];

	fprintf(stdout, "using image %s\n", imageFileName );

	image = rgbReadImageFile(imageFileName, &imageWidth,
				&imageHeight);

	auxGetScreenSize( &screenWidth, &screenHeight );
	windowWidth = screenWidth / 2;
	windowHeight = screenHeight / 2;
	aspect = (GLfloat)windowWidth / (GLfloat)windowHeight;

	auxInitPosition( screenWidth / 4, screenHeight / 4, windowWidth, 
		windowHeight );
	auxInitDisplayMode( AUX_RGBA | AUX_DOUBLE );
	auxInitWindow( argv[0] );

	auxAnimation( AUX_ON );

	auxReshapeFunc( resize );

	auxKeyFunc( AUX_ESCAPE, auxQuit );

	glClearColor( 0, 0, 0, 1 );

	perspectiveViewing( aspect );

	/* Note that the image must be 2^n by 2^m pixels or it
	 * will not appear as a texture. Could use gluScaleImage().
	 * If you don't see texture, check %imgsize filename 
	 */
	initTexture( image, imageWidth, imageHeight );
}

GLvoid initTexture( unsigned int *image, 
	GLsizei imageWidth, GLsizei imageHeight )
{
	/* Decal uses only the texture color, not the original polygon 
	 * color, unless there is alpha in texture, in which case
	 * alpha determines the percentage of texture blended with 
	 * the polygon color.
	 *
	 * Decal may be faster than the default modulate mode.
	 */
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
	
	/* Setting the minification and magnification filters
	 * to nearest instead of linear, may run faster on some platforms, 
	 * with possibly lower quality 
	 */
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER,
		GL_NEAREST);

	/* Since we're not using mipmaps, we must set the 
	 * minification filter to something other than the 
	 * default (GL_NEAREST_MIPMAP_LINEAR)
	 */
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
		GL_NEAREST);
	
	/* Load texture image to level 0 (original size) image must be 
	 * 2^m x 2^n.
	 *
	 * glTexImage2D( target, level, components, width, 
	 *		height, borderWidth, format,  type, imageArray ) 
	 */
	glTexImage2D(GL_TEXTURE_2D, 0, 4, imageWidth, imageHeight,
			 0, GL_RGBA, GL_UNSIGNED_BYTE, image);
	
	glEnable(GL_TEXTURE_2D);
}


GLvoid
resize( GLsizei width, GLsizei height )
{
	GLfloat     aspect;
	glViewport( 0, 0, width, height );

	aspect = (GLfloat) (width) / height;
	perspectiveViewing( aspect );
}

GLvoid
perspectiveViewing( GLfloat aspect )
{
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0, aspect, 1.0, 50.0 );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	glTranslatef( 0.0, 0.0, -12.0 ); 
}

GLvoid
drawScene(void)
{
	static GLfloat swim;

	static float v0[3] = {-1.5,-1.0,0.0};
	static float v1[3] = {1.5,-1.0,0.0};
	static float v2[3] = {1.5,1.0,0.0};
	static float v3[3] = {-1.5,1.0,0.0};
	
	static float t0[3] = {0.0, 0.0};
	static float t1[3] = {1.0, 0.0};
	static float t2[3] = {1.0, 1.0};
	static float t3[3] = {0.0, 1.0};

	glClear(GL_COLOR_BUFFER_BIT);
	
	/* set polygon color to black since in decal mode.  Otherwise
	 * color of polygon shows through sine in modulate mode.
	 */
	glColor4f( 0.0, 0.0, 0.0, 1.0 );
	glPushMatrix ();
		glTranslatef( -6.0 + swim, 0.2*sinf(swim * 3.0), -swim );
	
		glNormal3f(0,0,-1);
		glBegin (GL_QUADS);
			glTexCoord2fv(t0);
			glVertex3fv(v0);
			glTexCoord2fv(t1);
			glVertex3fv(v1);
			glTexCoord2fv(t2);
			glVertex3fv(v2);
			glTexCoord2fv(t3);
			glVertex3fv(v3);
		glEnd();

	glPopMatrix ();

	auxSwapBuffers();
	
	swim = fmodf( swim + 0.1, 35.0 );
}
