/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/* textures.c demonstrates storing textures in display lists.
 * If the user specifies an imagefile name when starting this 
 * program,  it will read in two textures and store them in 
 * display lists.  Else only one texture image will be used,  
 * and no display lists will be created.
 * Objects are then drawn using environment mapping.
 */
 
#include "aux.h"
#include <math.h>
#include <stdio.h>
/* rgbImageFile.h should be found in the ../../include directory */
#include "rgbImageFile.h"

/*  Function Prototypes  */

GLvoid  initialize( int, char ** );
GLvoid  drawScene( GLvoid );
GLvoid  perspectiveViewing( GLfloat );
GLvoid  resize( GLsizei, GLsizei );
GLvoid  initTexture( unsigned int *, GLsizei, GLsizei );

GLuint cvList, imageList;
GLboolean extraImage = GL_FALSE;

main ( int argc, char *argv[])
{
	initialize( argc,  argv );

	auxMainLoop( drawScene );
}

GLvoid
initialize( int argc, char *argv[] )
{
	char		*imageFileName;
	unsigned int 	*image;
	GLsizei		imageWidth, imageHeight;
	GLint		screenWidth, screenHeight;
	GLsizei		windowWidth, windowHeight;
	GLfloat		aspect;
	
	if (argc < 2) 
	{
		fprintf (stdout, "usage: %s <optional extraImageFile>\n", argv[0] );
	} 
	else
	{
		imageFileName = argv[1];
		extraImage = GL_TRUE;
		fprintf(stdout, "using image %s\n", imageFileName );
	}


	auxGetScreenSize( &screenWidth, &screenHeight );
	windowWidth =  screenWidth / 2;
	windowHeight = screenHeight / 2;
	aspect = (GLfloat)windowWidth / (GLfloat)windowHeight;

	auxInitPosition( screenWidth / 4, screenHeight / 4, windowWidth, 
		windowHeight );
	auxInitDisplayMode( AUX_RGBA | AUX_DOUBLE | AUX_DEPTH );
	auxInitWindow( argv[0] );

	auxAnimation( AUX_ON );

	auxReshapeFunc( resize );
	auxKeyFunc( AUX_ESCAPE, auxQuit );

	glEnable( GL_DEPTH_TEST );

	glClearColor( 0, 0, 0, 1 );
	glClear( GL_COLOR_BUFFER_BIT );
	auxSwapBuffers();
	fprintf( stdout,  "\nLoading textures\n");

	perspectiveViewing( aspect );


	if ( extraImage ) /* create display lists for both texture images */
	{
		imageList = glGenLists(1);
			image = rgbReadImageFile(imageFileName, &imageWidth,
					 &imageHeight);
			glNewList(imageList, GL_COMPILE);
			initTexture(image, imageWidth, imageHeight);
		glEndList();
		cvList = glGenLists(1);
			image = rgbReadImageFile("cv.rgb", &imageWidth, &imageHeight);
			glNewList(cvList, GL_COMPILE);
			initTexture(image, imageWidth, imageHeight);
		glEndList();
	}
	else	/* if no extra image, then just set only image now */
	{
		image = rgbReadImageFile("cv.rgb", &imageWidth, &imageHeight);
		initTexture(image, imageWidth, imageHeight);
	}
	
	/*
	 * enable automatic texture coordinate generation, 
	 * using environment (reflection) mapping.
	 */
	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
	glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);
	glEnable(GL_TEXTURE_2D);

}

GLvoid initTexture( unsigned int *image, 
	GLsizei imageWidth, GLsizei imageHeight )
{
	/* scale texture image, make mipmaps and load texture
	 *	gluBuild2DMipmaps( target, components, width, height,
	 *		format,  type, imageArray ) 
	 */
	gluBuild2DMipmaps(GL_TEXTURE_2D, 4, imageWidth, imageHeight,
			 GL_RGBA, GL_UNSIGNED_BYTE, image);
}

GLvoid
resize( GLsizei width, GLsizei height )
{
	GLfloat     aspect;
	glViewport( 0, 0, width, height );

	aspect = (GLfloat) (width) / height;
	perspectiveViewing( aspect );
}

GLvoid
perspectiveViewing( GLfloat aspect )
{
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0, aspect, 3.0, 15.0 );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	glTranslatef( 0.0, 0.0, -12.0 ); 
}

GLvoid
drawScene(void)
{
	static GLfloat spin;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glColor4f( 1.0, 1.0, 1.0, 1.0 );

	if ( extraImage )
		glCallList(cvList);
	glPushMatrix ();
		glRotatef(spin, 0.0, 1.0, 0.0);
		glTranslatef(3.0, 0.0, 0.0);

		glPushMatrix ();
			glRotatef(90.0, 1.0, 0.0, 0.0);
			glRotatef(-spin, 0.0, 1.0, 0.0);
			auxSolidCylinder( 1.0, 2.0 );	
		glPopMatrix ();
	glPopMatrix ();

	glPushMatrix ();
		glRotatef(spin + 90.0, 0.0, 1.0, 0.0);
		glTranslatef(3.0, 0.0, 0.0);

		glPushMatrix ();
			auxSolidSphere( 0.75 );	
		glPopMatrix ();
	glPopMatrix ();

	if ( extraImage )
		glCallList(imageList);
	glPushMatrix ();
		glRotatef(spin + 180.0, 0.0, 1.0, 0.0);
		glTranslatef(3.0, 0.0, 0.0);

		glPushMatrix ();
			glRotatef(spin, 0.0, 1.0, 0.0);
			auxSolidDodecahedron( 0.75 );	
		glPopMatrix ();
	glPopMatrix ();


	glPushMatrix ();
		glRotatef(spin + 270.0, 0.0, 1.0, 0.0);
		glTranslatef(3.0, 0.0, 0.0);

		glPushMatrix ();
			glRotatef( -3*spin, 0.0, 1.0, 0.0);
			auxSolidTorus( 0.25, 0.75 );	
		glPopMatrix ();
	glPopMatrix ();

	auxSwapBuffers();
	
	spin = fmodf( spin + 5.0, 360.0 );
}

